/* ========================================================================= //
# vi:  ts=4:
# vim: ts=4:
#?
#? NAME
#?      EnDeUser.js
#?
#? SYNOPSIS
#?      <SCRIPT language="JavaScript1.3" type="text/javascript" src="EnDeGUI.js"></SCRIPT>
#?      <SCRIPT language="JavaScript1.5" type="text/javascript" src="EnDeUser.js"></SCRIPT>
#?
#? DESCRIPTION
#?      This file contains user defined functions/methods used in  EnDe.js .
#?      It defines the  EnDeUser  class with following functions:
#?          .EN.vs      - fold viewstate (ASP.NET 1.x only, 2.x experimental)
#?          .EN.dispatch- dispatcher for user encoding functions
#?          .DE.vs      - parse viewstate (ASP.NET 1.x only, 2.x experimental)
#?          .DE.dispatch- dispatcher for user decoding functions
#?          .Check.dispatch - dispatcher for checksum functions
#?          .Check.quess- guessing checksums
#?          .DAT        - data structure for identifying data format
#?          .DAT.isXXX  - return true if given data is of format XXX
#?          .DAT.guess  - try to identify data format
#?          .IMG        - data structure for reading image data
#?          .IMG.isXXX  - return true if given data is of image format XXX
#?                       XXX: BMP, GIF, JPG, PNG, SVG, TXT, XPM
#?          .IMG.guess  - try to identify image format
#?          .IMG.getMIME64  - get MIME type prefix for base64 encoded image data
#?          .init       - initialize user data
#?
#? PRE-REQUESTS
#?      Some functions herein rely on variables and functions set elsewhere:
#?          EnDeGUI.alert()
#?          EnDeGUI.isKonqueror
#?
#?      If they are missing, somehow, they can simply be implemented as follows
#?      (which is the default anyway):
#?          EnDeGUI.alert    = function(f,s){alert(f+':'+s);}
#?          EnDeGUI.isKonqueror = false;
#?
#? SEE ALSO
#?      EnDe.js
#?      EnDeGUI.js
#?      EnDeFile.xml
#?      EnDeSamp.xml
#?
# HACKER's INFO
#       Simple debugging available if  userdebug  given in QUERY_STRING.
#
#? VERSION
#?      @(#) EnDeUser.js 3.8 11/04/29 11:43:48
#?
#? AUTHOR
#?      27-nov-07 Achim Hoffmann, mailto: EnDe (at) my (dash) stp (dot) net
#?
 * ========================================================================= */

// ========================================================================= //
// EnDeUser object methods                                                   //
// ========================================================================= //

var EnDeUser = new function() {
this.SID    = '3.8';
this.sid    = function() { return('@(#) EnDeUser.js 3.8 11/04/29 11:43:48 EnDeUser'); };

	// ===================================================================== //
	// global variables                                                      //
	// ===================================================================== //

this.ident  = '|   ';
this.pos    = 0;	// need a global, ugly ...
this.trace  = false;

	// ===================================================================== //
	// misc. methods                                                         //
	// ===================================================================== //

	// ===================================================================== //
	// viewstate functions                                                   //
	// ===================================================================== //

this.EN     = new function() {
  this.sid  = function() { return(EnDeUser.sid() + '.EN'); };
  this.vs   = function(src) {
  //#? convert data to ViewState 1.x format
	src = src.replace(/\|   /g,'');
	src = src.replace(/\n/g,'');
	return src;
  }; // vs

  this.dispatch = function(type,mode,uppercase,src,prefix,suffix,delimiter) {
  //#? dispatcher for user encoding functions
	_dpr(this.sid()+'.dispatch: '+type+'\t:uppercase='+uppercase+'\tprefix='+prefix+'\tsuffix='+suffix+'\tdelimiter='+delimiter);
	switch (type) {
	  case 'viewstate': return this.vs(src); break;
	}
	return null;
  }; // dispatch
}; // EN

this.DE     = new function() {
  this.sid  = function() { return(EnDeUser.sid() + '.DE'); };
  this.vs   = function(src) {
  //#? convert data from ViewState 1.x format
	if (EnDeGUI.isKonqueror){
		EnDeGUI.alert('** Sorry, Konqueror cannot use this function **');
		return src;
	}
	function _fff1(src) {
		// unicodes fff1 .. fffb are not used (in most browsers, hopefully),
		// hence we use them to mark own insertions
		return String.fromCharCode(65520+src);
	}
	function _mask(src) {
		// unicodes fff1 .. fffb are not used (in most browsers, hopefully),
		// hence we use them to mark own insertions
		switch (src) {
			  case 65520 : return(_fff1(1)  + 'Viewstate v1.0' + _fff1(11)); break;
			  case 65521 : return(_fff1(1)  + 'Viewstate v2.0' + _fff1(11)); break;
			  case '\x02': return(_fff1(2)  + 'Controlstate'   + _fff1(11)); break;
			  case '\x04': return(_fff1(4)  + 'Integer'        + _fff1(11)); break;
			  case '\x05': return(_fff1(5)  + 'String'         + _fff1(11)); break;
			  case '\x1e': return(_fff1(5)  + 'string'         + _fff1(11)); break; // ToDo: unsure
			  case '\x0f': return(_fff1(6)  + 'Pair'           + _fff1(11)); break;
			  case '\x16': return(_fff1(3)  + 'List'           + _fff1(11)); break;
			  case '\x17': return(_fff1(7)  + 'Arraylist'      + _fff1(11)); break;
			  case '\x18': return(_fff1(8)  + 'Hashlist'       + _fff1(11)); break;
			  case '\x66': return(_fff1(4)  + 'Int32'          + _fff1(11)); break; // ToDo: unsure
			  case '#'   : return(_fff1(10) + 'MAC'            + _fff1(11)); break;
			  case '['   : return(_fff1(10) + '['              + _fff1(11)); break;
			  case ']'   : return(_fff1(10) + ']'              + _fff1(11)); break;
			  default    : return(_fff1(9)  +  src             + _fff1(11)); break;
		}
		return src; // fallback
	}
	function _ident(i) { var kkk = ''; for (var k=0; k<i; k++, kkk += '|   '){}; return '\n'+kkk; }
	function _bytes(src,i,len) { return src.substr(i,len); }
	function _v1(src,i) {
		// parse Viewstate 1.x format
		// this code is build on reverse engeneering sample data
		var lev = 0;
		var bbb = '';
		var ccc = '';
		var kkk = '';
		var arr = null;
		var rex = null;
		var esc = 0;    // set to 1 if \ found, reset at next character
		var key = 0;    // incremented if < or ; decremented for [tpli]  and >
	                	// used to beautify idents
	    	key = 1;    // we assume that the viewstate's first character is a key
		while (i<src.length) {
			if (i>99999) { EnDeGUI.alert('viewstate too large'); return kkk; break; }
			ccc = src.charAt(i);
			i++;
			if (esc==1) {
				esc = 0;
				kkk += ccc;
				continue;
			}
			switch (ccc) {
			  case 'l'   :
			  case 't'   :
			  case 'p'   :
			  case 'i'   :
				if (key>0) {  // got a special key (tripple, tuplet)
					kkk += '\n' + bbb + ccc;
					key = -1;
				} else {
					kkk += ccc;
				}
				break;
			  case '\\': esc = 1; kkk += ccc; break;
			  case '<':
				lev++;
				if (key>=0) { kkk += '\n' + bbb; }
				//kkk += '\n' + bbb + ccc;
				kkk += ccc;
				key=1;
				/*
				 * limited JavaScript in Firefox (2.0.0.10), returns error
				 *    Error: overlarge minimum (67395) ....
				 * hence no sophisticated compact parsing for huge data
				 */
				if (src.length < 67390 ) {
					rex = new RegExp( '(?:^.{'+i+'})([a-zA-Z0-9.,;:/\(\)\\_-]+>)' );
					arr = src.match(rex);
					if (arr!=null) {
						kkk += arr[1];
						i += arr[1].length;
						arr.length = 0;
					} else {
						bbb += '|   ';
					}
				} else {
					bbb += '|   ';
				}
				break;
			  case '>':
				key=0;
				lev--;
				bbb  = bbb.substr(0,(bbb.length-4));
				kkk += '\n' + bbb + ccc;
				break;
			  case ';':
				key++;
				//kkk += '\n' + bbb + ccc;
				kkk += ccc;
				break;
			  default: kkk += ccc; key = 0; break;
			}
		}
		rex = null;
		return kkk;
	} // _v1

	function _v2(src,key,lev) {
	/* parse src starting at EnDeUser.pos */
		// parse Viewstate 2.x format
		// this ugly code is build on reverse engeneering sample data
		// ToDo: _v2() still incomplete and buggy
		var kkk = '';
		var bbb = '';
		var ccc = 0; // count items in list, hashlist, arraylist, ...
		var x   = 0;
		//#dbx if (lev>1){ alert(EnDeUser.pos+' \n'+src); };
		goab: while (EnDeUser.pos<src.length) {
			/* loop over data stream src (the binary viewstate)
			 * parsed data objects where each object will be identified by
			 * a leading byte (see switch statement) which is followed by the
			 * size of the object
			 * all objects can be used recursive
			 * base object types are "string" and "integer" (probably "byte")
			 */
			if (EnDeUser.pos>199999) { EnDeGUI.alert('viewstate too large'); return bbb; break; }
			switch (src[EnDeUser.pos]) {
/*
	case '\xff\x01':  // format 2.x viewstate
	case '\x01':  // t  - (Triplet) viewstate (sollte nur nach \xff komment)
	case '\x02':  // c  - controlstate
	case '\x03':  // l ??
	case '\x04':  // i 2-byte ??
	case '\x05':  // s  - naechstes Byte ist Laenge (in Bytes) ?? (in Hashtable Anzahl Name=Value Pairs)
	case '\x09':  //   ??
	case '\x0a':  //   ??
	case '\x0c':  //   ??
	case '\x0f\x0f':  // p  - pair
	case '\x0f\x0f\x64':  // MAC ??
	case '\x1b':  //   ??
	case '\x1e':  // ?  - ?unknown String? (naechstes Byte ist Laenge in Bytes)
	case '\x1f':  //   ??
	case '\x16':  //   ?? List - naechstes Byte Anzahl Eintraege
	case '\x17':  // a  - Arraylist (naechstes Byte Anzahl Elemente)
	case '\x18':  // h  - Hashlist
	case '\x63':  //   ??
	case '\x64':  //   ?? (Ende String?) Start MAC?
*/
	
			  case '\x64': bbb += _ident(lev) + _mask(src[EnDeUser.pos]); break;
			  case '\x02': bbb += _ident(lev) + _mask(src[EnDeUser.pos]); break;
			  case '\x04':
						// ToDo: integer probably 2 bytes fixed
			  case '\x1e': // unknown string (next byte is length)
			  case '\x05': // string (next byte is length)
					bbb += _ident(lev) + _mask(src[EnDeUser.pos]);
					EnDeUser.pos++;
					ccc = new Number(src.charCodeAt(EnDeUser.pos));
					bbb += '(' + ccc + ')';
					EnDeUser.pos++;
					kkk  = _bytes(src,EnDeUser.pos,ccc); EnDeUser.pos += kkk.length - 1;
					bbb += _ident(lev+1) + kkk;
					ccc = null;
					break;
			  case '\x0f': // pair
					EnDeUser.pos += 2;
					if (src[EnDeUser.pos] != '\x64') {
						bbb += _ident(lev) + _mask('\x0f'); // got a simple pair
						lev++;
						EnDeUser.pos--;
					} else { // seems to be final MAC
						bbb += _ident(lev) + _mask('#');
						bbb += _ident(lev) + '|' + _mask('[');
						lev++;
						EnDeUser.pos++;
						bbb += _v2(src,1,lev);
						bbb += _ident(lev) + '|' + _mask('[');
						bbb += _ident(lev+1) + _bytes(src,EnDeUser.pos,src.length);
						EnDeUser.pos = src.length;
						bbb += _ident(lev) + '|' + _mask(']');
						lev--;
						bbb += _ident(lev) + '|' + _mask(']');
					}
					if (key==1) { return bbb; } // don't continue loop
					break;
			  case '\x16': // list
			  case '\x17': // array (next byte number of elements)
					ccc = new Number(src.charCodeAt(EnDeUser.pos+1));
					bbb += _ident(lev) + _mask(src[EnDeUser.pos]) + '(' + ccc + ')';
					EnDeUser.pos += 2;
					//EnDeUser.pos += 3;
/*
					EnDeUser.pos++;
					ccc = new Number(src.charCodeAt(EnDeUser.pos));
					bbb += _ident(lev) + _mask(src[EnDeUser.pos]) + '(' + ccc + ')';
					EnDeUser.pos++;
*/
					lev++;
					for (x=0; x<ccc; x++) {
						bbb += _ident(lev-1) + '|' + _mask('[') + (x+1);
						bbb += _v2(src,1,lev);
						bbb += _ident(lev-1) + '|' + _mask(']');
					}
					lev--;
					ccc = null;
					if (key==1) { return bbb; } // don't continue loop
					continue; // read next byte
					break;
			  case '\x18': // hash
					ccc = new Number(src.charCodeAt(EnDeUser.pos+1));
					bbb += _ident(lev) + _mask(src[EnDeUser.pos]) + '(' + ccc + ')';
					EnDeUser.pos += 2;
					lev++;
					for (x=0; x<ccc; x++) {
						bbb += _ident(lev-1) + '|' + _mask('{') + (x+1);
						bbb += _v2(src,1,lev);
						bbb += _v2(src,1,lev);
						bbb += _ident(lev-1) + '|' + _mask('}');
					}
					ccc = null;
					if (key==1) { return bbb; } // don't continue loop
					continue; // read next byte
					break;
			  default:     bbb += src[EnDeUser.pos]; _dpr('def:'); break;
			} // switch
			EnDeUser.pos++;
			if (key==1) { return bbb; } // don't continue loop
		}  // while
		return bbb;
	}; // _v2
	var net = 0;    // version 1.x or version 2.x
	var ccc = 0;
	var lev = 0;
	var bux = '';
	while (ccc<src.length) { // detect viewstate, silently ignore anything before
		if (ccc>99999) { EnDeGUI.alert('viewstate too large'); break; }
		switch (src[ccc]) {
		  case '\uff01':                                    net = 2;    break;
		  case '\xff'  : if (src[ccc+1] == '\x01') { ccc++; net = 2; }; break; 
		  case 't'     : if (src[ccc+1] == '<')    { ccc++; net = 1; }; break; 
		}
		ccc++;
		if (net != 0) { break; } // found start of viewstate
		bux += src[ccc-1];
	}
	if (net != 1) {
		bux += _mask(65519+net);
		EnDeUser.pos = ccc;
	}
	if (net == 1) { bux += 't<\n' +_v1(src,ccc); } // t<  stripped above ..
	if (net == 2) { bux += _v2(src,0,0); }
	return bux;
  }; // vs

  this.dispatch = function(type,mode,uppercase,src,prefix,suffix,delimiter) {
  //#? dispatcher for user decoding functions
	_dpr(this.sid()+'.dispatch: '+type+'\t:uppercase='+uppercase+'\tprefix='+prefix+'\tsuffix='+suffix+'\tdelimiter='+delimiter);
	switch (type) {
	  case 'VSb64':     return this.vs( EnDe.DE.dispatch('base64',mode,'',src,prefix, '', '') ); break;
	  case 'VSraw':     return this.vs(src); break;
	  case 'viewstate': return this.vs(src); break; // for historical backward compatibility
	}
	return null;
  }; // dispatch
}; // DE

	// ===================================================================== //
	// checksum functions                                                    //
	// ===================================================================== //
	
this.Check  = new function() {
  this.sid  = function() { return(EnDeUser.sid() + '.Check'); };
  this.dispatch= function(tst) {
  //#? dispatcher for various checksum functions
	function $(id) { return document.getElementById(id); };
	var bux = '';
	var ccc = ' checksum: ';
	var src = $('EnDeDOM.DE.text').value;
        if ($('EnDeDOM.GUI.select').checked===true) {
		// use selection if there is one
		var kkk = EnDeGUI.selectionGet( $('EnDeDOM.EN.text') );
		if ((kkk!=null)&&(kkk!='')) { src = kkk; }
		kkk = null;
        }
	switch (tst) {
	  case 'getISBN' : bux = EnDe.Check.ISBN.get( src );    ccc = 'get ISBN'      + ccc; break;
	  case 'getIBAN' : bux = EnDe.Check.IBAN.get( src );    ccc = 'get IBAN'      + ccc; break;
	  case 'getLuhn' : bux = EnDe.Check.Luhn.get( src );    ccc = 'get Luhn'      + ccc; break;
	  case 'getUPC'  : bux = EnDe.Check.UPC.get( src );     ccc = 'get UPC '      + ccc; break;
	  case 'getEAN'  : bux = EnDe.Check.EAN.get( src );     ccc = 'get EAN '      + ccc; break;
	  case 'getGLN'  : bux = EnDe.Check.GLN.get( src );     ccc = 'get GLN '      + ccc; break;
	  case 'getEFT'  : bux = EnDe.Check.EFT.get( src );     ccc = 'get EFT '      + ccc; break;
	  case 'getD5'   : bux = EnDe.Check.D5.get( src );      ccc = 'get D5  '      + ccc; break;
	  case 'isISBN'  : bux = EnDe.Check.ISBN.is( src );     ccc = 'check ISBN'    + ccc; break;
	  case 'isIBAN'  : bux = EnDe.Check.IBAN.is( src );     ccc = 'check IBAN'    + ccc; break;
	  case 'isLuhn'  : bux = EnDe.Check.Luhn.is( src );     ccc = 'check Luhn'    + ccc; break;
	  case 'isUPC'   : bux = EnDe.Check.UPC.is( src );      ccc = 'check UPC '    + ccc; break;
	  case 'isEAN'   : bux = EnDe.Check.EAN.is( src );      ccc = 'check EAN '    + ccc; break;
	  case 'isGLN'   : bux = EnDe.Check.GLN.is( src );      ccc = 'check GLN '    + ccc; break;
	  case 'isEFT'   : bux = EnDe.Check.EFT.is( src );      ccc = 'check EFT '    + ccc; break;
	  case 'isD5'    : bux = EnDe.Check.D5.is( src );       ccc = 'check D5  '    + ccc; break;
	  case 'valISBN' : bux = EnDe.Check.ISBN.valid( src );  ccc = 'validate ISBN' + ccc; break;
	  case 'valIBAN' : bux = EnDe.Check.IBAN.valid( src );  ccc = 'validate IBAN' + ccc; break;
	  case 'valLuhn' : bux = EnDe.Check.Luhn.valid( src );  ccc = 'validate Luhn' + ccc; break;
	  case 'valUPC'  : bux = EnDe.Check.UPC.valid( src );   ccc = 'validate UPC ' + ccc; break;
	  case 'valEAN'  : bux = EnDe.Check.EAN.valid( src );   ccc = 'validate EAN ' + ccc; break;
	  case 'valGLN'  : bux = EnDe.Check.GLN.valid( src );   ccc = 'validate GLN ' + ccc; break;
	  case 'valEFT'  : bux = EnDe.Check.EFT.valid( src );   ccc = 'validate EFT ' + ccc; break;
	  case 'valD5'   : bux = EnDe.Check.D5.valid( src );    ccc = 'validate D5  ' + ccc; break;
	  case 'isSSN'   : bux = EnDe.Check.SSN.is( src );      ccc = 'check SSN    ' + ccc; break;
	  case 'guess'   : EnDeGUI.cont(EnDeUser.Check.guess( src )); return false;          break;
	}
	prompt(ccc, bux);
	bux = null;
	return false;
  }; // dispatch
  
  this.guess     = function(src) {
  //#? guessing checksums
	var bux = '<table border="1"><tr><th colspan="4">' + src + '</th></tr>';
	bux += '<tr><td>&#160;</td><td>get()'   + '</td><td>' +'is()'                  + '</td><td>' +'valid()'                  + '</td></tr>';
	bux += '<tr><th>Luhn</th>';
	bux += '<td>' + EnDe.Check.Luhn.get(src) + '</td><td>' + EnDe.Check.Luhn.is(src) + '</td><td>' + EnDe.Check.Luhn.valid(src) + '</td></tr>';
	bux += '<tr><th>IBAN</th>';
//	bux += '<td>' + EnDe.Check.IBAN.get(src) + '</td><td>' + EnDe.Check.IBAN.is(src) + '</td><td>' + EnDe.Check.IBAN.valid(src) + '</td></tr>';
	bux += '<td>' + '??' + '</td><td>' + '??' + '</td><td>' + '??' + '</td></tr>';
	bux += '<tr><th>ISBN</th>';
	bux += '<td>' + EnDe.Check.ISBN.get(src) + '</td><td>' + EnDe.Check.ISBN.is(src) + '</td><td>' + EnDe.Check.ISBN.valid(src) + '</td></tr>';
	bux += '<tr><th>EFT</th>';
	bux += '<td>' + EnDe.Check.EFT.get(src)  + '</td><td>' + EnDe.Check.EFT.is(src)  + '</td><td>' + EnDe.Check.EFT.valid(src)  + '</td></tr>';
	bux += '<tr><th>UPC</th>';
	bux += '<td>' + EnDe.Check.UPC.get(src)  + '</td><td>' + EnDe.Check.UPC.is(src)  + '</td><td>' + EnDe.Check.UPC.valid(src)  + '</td></tr>';
	bux += '<tr><th>EAN</th>';
	bux += '<td>' + EnDe.Check.EAN.get(src)  + '</td><td>' + EnDe.Check.EAN.is(src)  + '</td><td>' + EnDe.Check.EAN.valid(src)  + '</td></tr>';
	bux += '<tr><th>GLN</th>';
	bux += '<td>' + EnDe.Check.GLN.get(src)  + '</td><td>' + EnDe.Check.GLN.is(src)  + '</td><td>' + EnDe.Check.GLN.valid(src)  + '</td></tr>';
	bux += '<tr><th>Verhoeff</th>';
	bux += '<td>' + EnDe.Check.D5.get(src)   + '</td><td>' + EnDe.Check.D5.is(src)   + '</td><td>' + EnDe.Check.D5.valid(src)   + '</td></tr>';
	bux += '<tr><th>GLN, ILN</th><td colspan="3">' + EnDe.Check.EAN.code(src) + '</td></tr>';
	bux += '<tr><th>EAN, IAN</th><td colspan="3">' + EnDe.Check.GLN.code(src) + '</td></tr>';
	bux += '<tr><th>IIN</th><td colspan="3">' + EnDe.Check.CC.get(src) + '</td></tr>';
	bux += '<tr><th>SSN</th><td colspan="3">' + EnDe.Check.SSN.is(src) + '</td></tr>';
	bux += '</table>';
	return(bux);
  }; // guess

}; // Check

this.DAT    = new function() {
  this.sid  = function() { return(EnDeUser.sid() + '.DAT'); };

  this.isGZ     = function(src) {
  //#? check if data is gzip compressed data
	return (src.substr(0,12).match(/^\x1f\x8b\x08.....\x02\x03\xec/)===null) ? false : true;
  }; // isGZ

  this.isSWF    = function(src) {
  //#? check if data is Adobe SWF file
	return (src.substr(0,12).match(/^[FC]WS[\x04-\x09]/)===null) ? false : true;
  }; // isSWF

  this.isZIP    = function(src) {
  //#? check if data is ZIP archive
	return (src.substr(0,12).match(/^PK\x03\x04[\x09-\x14]/)===null) ? false : true;
  }; // isZIP

  this.guess    = function(src) {
  //#? try to identify data format
	if (this.isGZ( src)===true) { return 'gz';  }
	if (this.isSWF(src)===true) { return 'swf'; }
	if (this.isZIP(src)===true) { return 'zip'; }
	return '';
  }; // guess

  this.type     = function(src) {
  //#? return human readable string for data format
	switch(src) {
	  case 'gz':  return 'gzip compressed data'; break;
	  case 'swf': return 'Adobe SWF (flash) file'; break;
	  case 'zip': return 'ZIP archive'; break;
	}
	return '';
  }; // guess

}; // DAT

this.IMG    = new function() {
  this.sid  = function() { return(EnDeUser.sid() + '.IMG'); };

  this.isBMP    = function(src) {
  //#? check if data is image in BMP format (lazy check only!)
	return (src.substr(0,12).match(/^BM..\x00\x00\x00\x00/)===null) ? false : true;
  }; // isBMP

  this.isTXT    = function(src) {
  //#? check if data is image in BMP format as C data structure
	return (src.substr(0,99).match(/#define\s+.*table_/)===null) ? false : true;
  }; // isTXT

  this.isPNG    = function(src) {
  //#? check if data is image in PNG format (lazy check only!)
	return (src.substr(0,12).match(/^.PNG[\r\n]/)===null) ? false : true;
	// we use a lazy check, following more strict
	//return (src.substr(0,12).match(/^\x89PNG\r\n/)===null) ? false : true;
  }; // isPNG

  this.isGIF    = function(src) {
  //#? check if data is image in GIF format (lazy check only!)
	return (src.substr(0,12).match(/^GIF89a/)===null) ? false : true;
  }; // isGIF

  this.isJPG    = function(src) {
  //#? check if data is image in JPG format (lazy check only!)
	return (src.substr(0,12).match(/^.....\x10JFIF/)===null) ? false : true;
  }; // isJPG

  this.isSVG    = function(src) {
  //#? check if data is image in SVG format (lazy check only!)
	return (src.substr(0,12).match(/DOCTYPE\s+svg/i)===null) ? false : true;
  }; // isSVG

  this.isXPM    = function(src) {
  //#? check if data is image in XPM format (lazy check only!)
// /* XPM */
// static char * ssh_xpm[] = 
	return false;
  }; // isXPM

  this.guess    = function(src) {
  //#? try to identify image format
	if (this.isGIF(src)===true) { return 'gif'; }
	if (this.isJPG(src)===true) { return 'jpg'; }
	if (this.isPNG(src)===true) { return 'png'; }
	if (this.isBMP(src)===true) { return 'bmp'; }
	if (this.isSVG(src)===true) { return 'svg'; }
	if (this.isTXT(src)===true) { return 'txt'; }
	if (this.isXPM(src)===true) { return 'xpm'; }
	return '';
  }; // guess

  this.getMIME  = function(src) {
  //#? get MIME type prefix for image data (returns data:image/x-icon for unknown format)
	var bux = 'data:image/';
	switch (src.toLowerCase()) {
	  case 'bmp': bux += 'bmp'; break;
	  case 'txt': bux += 'bmp'; break;
	  case 'gif': bux += 'gif'; break;
	  case 'jpg': bux += 'jpg'; break;
	  case 'png': bux += 'png'; break;
	  case 'svg': bux += 'svg'; break;
	  default   : bux += 'x-icon'; break;
	}
	return bux + ',';
  }; // isMIME

  this.getMIME64= function(src) {
  //#? get MIME type prefix for base64 encoded image data
	return this.getMIME(src).replace(/,$/, ';base64,');
  }; // isMIME64

}; // IMG
  
this.init      = function() {
//#? initialize user data
}; // init

}; // EnDeUser
